@@ -3,6 +3,7 @@ package Travel::Routing::DE::EFA;
 use strict;
 use warnings;
 use 5.010;
+use utf8;
 
 no if $] >= 5.018, warnings => "experimental::smartmatch";
 
@@ -41,7 +42,7 @@ use Exception::Class (
 	},
 );
 
-our $VERSION = '2.16';
+our $VERSION = '2.19';
 
 sub set_time {
 	my ( $self, %conf ) = @_;
@@ -93,6 +94,13 @@ sub date {
 
 	my ( $day, $month, $year ) = split( /[.]/, $date );
 
+	if ( $date eq 'tomorrow' ) {
+		( undef, undef, undef, $day, $month, $year )
+		  = localtime( time + 86400 );
+		$month += 1;
+		$year  += 1900;
+	}
+
 	if (
 		not(    defined $day
 			and length($day)
@@ -832,13 +840,15 @@ sub check_ambiguous_xml {
 	if ( $s_place eq 'notidentified' ) {
 		Travel::Routing::DE::EFA::Exception::Setup->throw(
 			option => 'place',
-			error  => 'unknown place (typo?)'
+			error  => 'unknown place',
+			have   => ( $e_place->findnodes($xp_place_input) )[0]->textContent,
 		);
 	}
 	if ( $s_name eq 'notidentified' ) {
 		Travel::Routing::DE::EFA::Exception::Setup->throw(
 			option => 'name',
-			error  => 'unknown name (typo?)'
+			error  => 'unknown name',
+			have   => ( $e_name->findnodes($xp_name_input) )[0]->textContent,
 		);
 	}
 
@@ -855,99 +865,119 @@ sub routes {
 
 # static
 sub get_efa_urls {
+
+	# sorted lexically by shortname
 	return (
 		{
-			url       => 'http://www.ding.eu/ding3/XSLT_TRIP_REQUEST2',
+			url       => 'https://bsvg.efa.de/bsvagstd/XML_TRIP_REQUEST2',
+			name      => 'Braunschweiger Verkehrs-GmbH',
+			shortname => 'BSVG',
+		},
+		{
+			url       => 'https://www.ding.eu/ding3/XSLT_TRIP_REQUEST2',
 			name      => 'Donau-Iller Nahverkehrsverbund',
 			shortname => 'DING',
 		},
 		{
-			url       => 'http://efa.ivb.at/ivb/XSLT_TRIP_REQUEST2',
-			name      => 'Innsbrucker Verkehrsbetriebe',
-			shortname => 'IVB',
+			url  => 'https://projekte.kvv-efa.de/sl3-alone/XSLT_TRIP_REQUEST2',
+			name => 'Karlsruher Verkehrsverbund',
+			shortname => 'KVV',
 		},
+		{
+			url       => 'https://www.linzag.at/static/XSLT_TRIP_REQUEST2',
+			name      => 'Linz AG',
+			shortname => 'LinzAG',
+		},
+		{
+			url  => 'https://efa.mvv-muenchen.de/mobile/XSLT_TRIP_REQUEST2',
+			name => 'Münchner Verkehrs- und Tarifverbund',
+			shortname => 'MVV',
+		},
+		{
+			url       => 'https://www.efa-bw.de/nvbw/XSLT_TRIP_REQUEST2',
+			name      => 'Nahverkehrsgesellschaft Baden-Württemberg',
+			shortname => 'NVBW',
+		},
+
+		# HTTPS not supported
 		{
 			url       => 'http://efa.svv-info.at/sbs/XSLT_TRIP_REQUEST2',
 			name      => 'Salzburger Verkehrsverbund',
 			shortname => 'SVV',
 		},
-		{
-			url       => 'http://efa.vor.at/wvb/XSLT_TRIP_REQUEST2',
-			name      => 'Verkehrsverbund Ost-Region',
-			shortname => 'VOR',
-		},
 
-		# Returns broken Unicode which makes Encode::decode die()
-		#{
-		#	url  => 'http://fahrplan.verbundlinie.at/stv/XSLT_TRIP_REQUEST2',
-		#	name => 'Verkehrsverbund Steiermark',
-		#	shortname => 'Verbundlinie',
-		#},
-		{
-			url       => 'http://www.linzag.at/static/XSLT_TRIP_REQUEST2',
-			name      => 'Linz AG',
-			shortname => 'LinzAG',
-		},
-		{
-			url       => 'http://212.114.197.7/vgnExt_oeffi/XML_TRIP_REQUEST2',
-			name      => 'Verkehrsverbund Grossraum Nuernberg',
-			shortname => 'VGN',
-		},
-		{
-			url       => 'http://efa.vrr.de/vrr/XSLT_TRIP_REQUEST2',
-			name      => 'Verkehrsverbund Rhein-Ruhr',
-			shortname => 'VRR',
-		},
-		{
-			url       => 'http://app.vrr.de/vrrstd/XML_TRIP_REQUEST2',
-			name      => 'Verkehrsverbund Rhein-Ruhr (alternative)',
-			shortname => 'VRR2',
-		},
-		{
-			url       => 'http://www2.vvs.de/vvs/XSLT_TRIP_REQUEST2',
-			name      => 'Verkehrsverbund Stuttgart',
-			shortname => 'VVS',
-		},
-		{
-			url => 'http://delfi1.vvo-online.de:8080/delfi3/XSLT_TRIP_REQUEST2',
-			name      => 'Verkehrsverbund Oberelbe',
-			shortname => 'VVO',
-		},
-		{
-			url       => 'http://delfi.vrn.de/delfi/XSLT_TRIP_REQUEST2',
-			name      => 'Verkehrsverbund Rhein-Neckar (DELFI)',
-			shortname => 'VRNdelfi',
-		},
-		{
-			url       => 'http://fahrplanauskunft.vrn.de/vrn/XML_TRIP_REQUEST2',
-			name      => 'Verkehrsverbund Rhein-Neckar',
-			shortname => 'VRN',
-		},
-		{
-			url       => 'http://80.146.180.107/vmv/XSLT_TRIP_REQUEST2',
-			name      => 'Verkehrsgesellschaft Mecklenburg-Vorpommern',
-			shortname => 'VMV',
-		},
-		{
-			url       => 'http://213.144.24.66/kvv2/XML_TRIP_REQUEST2',
-			name      => 'Karlsruher Verkehrsverbund',
-			shortname => 'KVV',
-		},
+		# HTTPS: invalid certificate
 		{
 			url =>
 			  'http://www.travelineeastmidlands.co.uk/em/XSLT_TRIP_REQUEST2',
 			name      => 'Traveline East Midlands',
 			shortname => 'TLEM',
 		},
+		{
+			url       => 'https://efa.vagfr.de/vagfr3/XSLT_TRIP_REQUEST2',
+			name      => 'Freiburger Verkehrs AG',
+			shortname => 'VAG',
+		},
+
+		# HTTPS: unsupported protocol
 		{
 			url       => 'http://mobil.vbl.ch/vblmobil/XML_TRIP_REQUEST2',
 			name      => 'Verkehrsbetriebe Luzern',
 			shortname => 'VBL',
 		},
+
+		# HTTPS not supported
 		{
-			url       => 'http://bsvg.efa.de/bsvagstd/XML_TRIP_REQUEST2',
-			name      => 'Braunschweiger Verkehrs-GmbH',
-			shortname => 'BSVG',
+			url  => 'http://fahrplan.verbundlinie.at/stv/XSLT_TRIP_REQUEST2',
+			name => 'Verkehrsverbund Steiermark',
+			shortname => 'Verbundlinie',
+		},
+		{
+			url       => 'https://efa.vgn.de/vgnExt_oeffi/XML_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Grossraum Nuernberg',
+			shortname => 'VGN',
+		},
+
+		# HTTPS: certificate verification fails
+		{
+			url       => 'http://efa.vmv-mbh.de/vmv/XML_TRIP_REQUEST2',
+			name      => 'Verkehrsgesellschaft Mecklenburg-Vorpommern',
+			shortname => 'VMV',
+		},
+		{
+			url       => 'https://efa.vor.at/wvb/XSLT_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Ost-Region',
+			shortname => 'VOR',
+			encoding  => 'iso-8859-15',
+		},
+
+		# HTTPS not spported
+		{
+			url       => 'http://fahrplanauskunft.vrn.de/vrn/XML_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Rhein-Neckar',
+			shortname => 'VRN',
+		},
+		{
+			url       => 'https://efa.vrr.de/vrr/XSLT_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Rhein-Ruhr',
+			shortname => 'VRR',
+		},
+		{
+			url       => 'https://app.vrr.de/vrrstd/XML_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Rhein-Ruhr (alternative)',
+			shortname => 'VRR2',
+		},
+
+		# HTTPS not supported
+		{
+			url       => 'http://efa.vvo-online.de:8080/dvb/XSLT_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Oberelbe',
+			shortname => 'VVO',
+		},
+		{
+			url       => 'https://www2.vvs.de/vvs/XSLT_TRIP_REQUEST2',
+			name      => 'Verkehrsverbund Stuttgart',
+			shortname => 'VVS',
 		},
 	);
 }
@@ -984,7 +1014,7 @@ Travel::Routing::DE::EFA - unofficial interface to EFA-based itinerary services
 
 =head1 VERSION
 
-version 2.16
+version 2.19
 
 =head1 DESCRIPTION
 
@@ -1079,7 +1109,7 @@ Journey start time.  Default: now
 
 =item B<date> => I<DD.MM.>[I<YYYY>]
 
-Journey date.  Default: today
+Journey date.  Also accepts the string B<tomorrow>.  Default: today
 
 =item B<exclude> => \@exclude
 
@@ -1255,8 +1285,8 @@ None known.
 
 =head1 AUTHOR
 
-Copyright (C) 2009-2015 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
+Copyright (C) 2009-2020 by Daniel Friesel E<lt>derf@finalrewind.orgE<gt>
 
 =head1 LICENSE
 
-  0. You just DO WHAT THE FUCK YOU WANT TO.
+This program is licensed under the same terms as Perl itself.
